defmodule Pleroma.Web.Router do
  use Pleroma.Web, :router

  alias Pleroma.{Repo, User, Web.Router}

  def user_fetcher(username) do
    {:ok, Repo.get_by(User, %{nickname: username})}
  end

  pipeline :api do
    plug :accepts, ["json"]
    plug :fetch_session
    plug Pleroma.Plugs.AuthenticationPlug, %{fetcher: &Router.user_fetcher/1, optional: true}
  end

  pipeline :authenticated_api do
    plug :accepts, ["json"]
    plug :fetch_session
    plug Pleroma.Plugs.AuthenticationPlug, %{fetcher: &Router.user_fetcher/1}
  end

  pipeline :well_known do
    plug :accepts, ["xml", "xrd+xml"]
  end

  scope "/api", Pleroma.Web do
    pipe_through :api

    get "/help/test", TwitterAPI.UtilController, :help_test
    get "/statusnet/config", TwitterAPI.UtilController, :config

    get "/statuses/public_timeline", TwitterAPI.StatusController, :public_timeline
    get "/statuses/public_and_external_timeline", TwitterAPI.StatusController, :public_and_external_timeline
    get "/statuses/networkpublic_timeline", TwitterAPI.StatusController, :public_and_external_timeline
    get "/statuses/user_timeline", TwitterAPI.StatusController, :user_timeline

    get "/statuses/show/:id", TwitterAPI.StatusController, :fetch_status
    get "/statusnet/conversation/:id", TwitterAPI.StatusController, :fetch_conversation

    post "/account/register", TwitterAPI.UserController, :register
  end

  scope "/api", Pleroma.Web do
    pipe_through :authenticated_api

    get "/account/verify_credentials", TwitterAPI.UserController, :verify_credentials
    post "/account/verify_credentials", TwitterAPI.UserController, :verify_credentials

    get "/statuses/home_timeline", TwitterAPI.StatusController, :friends_timeline
    get "/statuses/friends_timeline", TwitterAPI.StatusController, :friends_timeline
    get "/statuses/mentions", TwitterAPI.StatusController, :mentions_timeline
    get "/statuses/mentions_timeline", TwitterAPI.StatusController, :mentions_timeline

    post "/statuses/update", TwitterAPI.StatusController, :status_update
    post "/statuses/retweet/:id", TwitterAPI.StatusController, :retweet

    post "/friendships/create", TwitterAPI.UserController, :follow
    post "/friendships/destroy", TwitterAPI.UserController, :unfollow

    post "/statusnet/media/upload", TwitterAPI.MediaController, :upload
    post "/media/upload", TwitterAPI.MediaController, :upload_json

    post "/favorites/create/:id", TwitterAPI.StatusController, :favorite
    post "/favorites/create", TwitterAPI.StatusController, :favorite
    post "/favorites/destroy/:id", TwitterAPI.StatusController, :unfavorite

    post "/qvitter/update_avatar", TwitterAPI.UserController, :update_avatar
  end

  pipeline :ostatus do
    plug :accepts, ["xml", "atom"]
  end

  scope "/", Pleroma.Web do
    pipe_through :ostatus

    get "/objects/:uuid", OStatus.OStatusController, :object

    get "/users/:nickname/feed", OStatus.OStatusController, :feed
    get "/users/:nickname", OStatus.OStatusController, :feed_redirect
    post "/users/:nickname/salmon", OStatus.OStatusController, :salmon_incoming
    post "/push/hub/:nickname", Websub.WebsubController, :websub_subscription_request
    get "/push/subscriptions/:id", Websub.WebsubController, :websub_subscription_confirmation
    post "/push/subscriptions/:id", Websub.WebsubController, :websub_incoming
  end

  scope "/.well-known", Pleroma.Web do
    pipe_through :well_known

    get "/host-meta", WebFinger.WebFingerController, :host_meta
    get "/webfinger", WebFinger.WebFingerController, :webfinger
  end

  scope "/", Fallback do
    get "/*path", RedirectController, :redirector
  end

end

defmodule Fallback.RedirectController do
  use Pleroma.Web, :controller
  def redirector(conn, _params), do: (if Mix.env != :test, do: send_file(conn, 200, "priv/static/index.html"))
end
